#
# A demonstration Tcl extension for teaish.
#
teaish-pkginfo-set {
  -name hello
  -name.dist hello-teaish
  -version 0.1
  -vsatisfies {{Tcl 8.5-}}
  -url {https://fossil.wanderinghorse.net/r/teaish}
  -pragmas {no-full-dist}
  -options {
    hello=1     => {Disable hello, world!}
  }
  -pkgInit.tcl sayhello.tcl
  -src {generic/hello.c}
}
#
# That ^^^ is technically all that's needed. The rest is mostly
# testing of teaish itself.
#

#teaish-pkginfo-set -vsatisfies {{Tcl 8.6-}}; # just for teaish testing
#teaish-pkginfo-set -pragmas {static-pkgIndex.tcl}; # just teaish testing

# Gets called by teaish-configure-core after bootstrapping is complete.
proc teaish-options {} {
  # This ordering is important for the --help text
  options-add {
    features=0  => {Enable various teaish feature tests and new --flags}
  }
  if {[teaish-argv-has --features --enable-features]} {
    use teaish/feature
    teaish-import-features -pre openssl
  }
  #options-add { hello=0 => {error - overridden} }; # just testing
  return {
    the-final-flag => {Just for demonstration's sake.}
  }
}

proc dummy-feature-check-fail {} {
  teaish-check-cached "Dummy feature test (always fails)" {
    return 0
  }
}
proc dummy-feature-check-pass {} {
  teaish-check-cached "Dummy feature test (always passes)" {
    return 1
  }
}

# Gets called by teaish-configure-core after processing of --flags.
proc teaish-configure {} {
  #proj-error just testing
  #proj-fatal just testing
  proj-assert {"" ne [teaish-pkginfo-get -src]}
  #teaish-src-add -dir -dist generic/hello.c
  teaish-define-to-cflag -quote TEAISH_PKGNAME TEAISH_VERSION
  teaish-define-to-cflag TEAISH_LOAD_PREFIX

  if {[opt-bool features]} {
    teaish-import-features ncurses
    define-push {TEAISH_LDFLAGS TEAISH_CFLAGS} {
      # These are just for testing teaish-feature-tests
      for {set i 0} {$i < 2} {incr i} {
        puts "******************** Feature tests..."
        foreach p {
          teaish-check-stdint
          teaish-check-libz
          teaish-check-librt
          teaish-check-dlopen
          teaish-check-libmath
          teaish-check-ncurses
          dummy-feature-check-fail
          dummy-feature-check-pass
        } {
          # Check that the caching works
          $p
        }
      }
      proj-assert {0 == [teaish-feature-cache-check 0 foo]}
      teaish-check-cached -key foo -quiet "Testing teaish-check-cached flags" {
        msg-result "Should not emit anything"
        expr 1
      }
      proj-assert {1 == [teaish-feature-cache-check 0 foo]}
      unset foo

      teaish-ldflags-prepend -lbar
      teaish-ldflags-add -prepend -lfoo
      teaish-ldflags-add -lbaz
      if {[teaish-is-mingw]} {
        teaish-cflags-add -DBROKEN_MINGW_CMDLINE
        teaish-ldflags-prepend -lkernel32 -lws2_32
      }

      msg-result "TEAISH_LDFLAGS = [get-define TEAISH_LDFLAGS]"
      msg-result "TEAISH_CFLAGS = [get-define TEAISH_CFLAGS]"
      msg-result "Clearing all of those flags, they were just for testing feature detection."
    }
  }; # testing of feature detection

  if {0} {
    apply {{} {
      teaish-pkginfo-get pi
      puts "teaish-pkginfo-get pi =>"
      parray pi
      proj-assert {[array size pi] > 6}
      proj-assert {"hello" eq [teaish-pkginfo-get -name]}
      puts "teaish-get -dir = [teaish-get -dir]"
      set v [teaish-pkginfo-get]
      puts "teaish-pkginfo-get = {$v}"
      #array set a $v
      #parray a
    }}
  }; # random other testing

  #puts "*** [proj-define-to-cflag TEAISH_LIBDIR_NAME TEAISH_LOAD_PREFIX]"
  #puts "*** [proj-define-to-cflag -list TEAISH_LIBDIR_NAME TEAISH_LOAD_PREFIX]"
  #puts "??? real tcl? [info exists ::tcl_library] $::autosetup(istcl)"
  #parray ::tcl_platform

  if {0} {
    # Just testing...
    set d [teaish-get -dir]
    teaish-make-obj -clean -recipe \
      [join [list \$(CC.tcl) -c -o \$@ $d/generic/hello.c]] \
      -deps config.log \
      -src $d/generic/hello.c \
      foo.o
    teaish-make-clean bar.o foo.z
  }

}
