# This is a teaish extension configuration file for building the
# TclTls project: https://core.tcl-lang.org/tcltls
#
teaish-pkginfo-set {
  -name tls
  -version 2.0b1
  -url {https://core.tcl-lang.org/tcltls}
  -vsatisfies {}
}
#  -pragmas {no-vsatisfies-error}

proc teaish-options {} {
  use teaish/feature
  teaish-import-features openssl; # adds --with-openssl flag
  return {

    with-tcltls:DIR
      => {Path to a checked-out copy of tcltls. Alternatively, set the
          TCLTLS_HOME environment variable to that dir instead of using
          this flag.}

    deprecation-warnings=0
      => {Enable warnings about usage of deprecated OpenSSL APIs.}
  }
}

proc teaish-configure {} {
  set isCanon [file exists [file join [teaish-get -dir] generic tls.c]]
  # ^^^ true if we're building directly from a teaish-ized copy of TclTLS
  if {$isCanon} {
    msg-result "Building from teaish-ized TclTLS source tree."
    set dirTls [teaish-get -dir]
  } else {
    msg-result "Building TclTLS from custom teaish-ized build tree."
    set dirTls [opt-val with-tcltls [proj-get-env TCLTLS_HOME]]
    if {![file isdirectory $dirTls]} {
      proj-indented-notice -error [subst {
        Cannot find --with-tcltls dir. $dirTls

        Use --with-tcltls=DIR to specify the directory containing
        a checkout of TclTLS:

        https://core.tcl-lang.org/tcltls

        Alternatively, create a file named .env-TCLTLS_HOME containing
        only the path to a checkout of tcltls, and the configure script
        will use that value if none other is specified.  Another option
        is to set the TCLTLS_HOME environment variable to that dir.
      }]
    }
  }
  foreach f {manifest.uuid library/tls.tcl} {
    if {![file exists $dirTls/$f]} {
      proj-indented-notice -error [subst {
        tcltls dir ($dirTls) does not look like a tcltls
        checkout.
      }]
    }
  }

  if {![teaish-check-openssl]} {
    proj-indented-notice -error {
      Did not find OpenSSL. Use --with-openssl=... to specify an
      OpenSSL and see --help for its options.
    }
  } elseif {0 == [proj-opt-truthy deprecation-warnings]} {
    # omit openssl deprecation warnings
    teaish-cflags-add -Wno-deprecated-declarations
  }

  define TCLTLS_HOME $dirTls
  if {!$isCanon} {
    #
    # Update some state from the upstream tree. This needs to come
    # pretty early because it has to update some
    # necessarily-already-calculated package state.
    #
    apply {{dir} {
      # Extract version number
      set version ""
      foreach line [proj-file-content $dir/configure.ac] {
        if {[string match {AC_INIT*[0-9]*} $line]} {
          set line [split $line {\[}]
          set line [lindex $line 2]
          set version [lindex [split $line {\]}] 0]
          break
        }
      }
      if {"" eq $version} {
        proj-fatal "Cannot extract TclTLS version from $dir/configure.ac"
      } elseif {[teaish-get -version] ne $version} {
        teaish-pkginfo-set -vars {
          -version $version
          -libDir tls$version
        }
        msg-result "Updated version to $version from $dir/configure.ac"
      }
    }} $dirTls
  }

  #
  # Set up the object files...
  #
  foreach c [glob $dirTls/generic/*.c] {
    #set c [relative-path $c $dirTls]
    set o [file rootname [file tail $c]].o
    teaish-make-obj $o $c; # add makefile code for compiling $c
    teaish-src-add  $o   ; # add it to the DLL
  }

  teaish-cflags-for-tea -I. -I$::autosetup(srcdir); # also adds -DPACKAGE_NAME/VERSION
  teaish-dist-add README.md
  define TCLTLS_IS_CANON $isCanon
  define TCLTLS_IS_IN_OWN_TREE [expr {$::autosetup(srcdir) eq $::autosetup(builddir)}]
}
